"use strict";(self.webpackChunkpackage_edit=self.webpackChunkpackage_edit||[]).push([[690],{6037:(t,e,i)=>{i.r(e),i.d(e,{default:()=>u});var c=function(){var t=this,e=t._self._c;return e("div",{staticClass:"agent-activity",class:[{"is-deleted":t.activity.meta.to_delete},"activity-scheme-"+t.activity.scheme]},[e("div",{staticClass:"clickable-text",class:{clicked:t.activity.meta.is_editing},on:{click:t.toggleEdit}},[e("span",[t._v(t._s(t.activity.activity))]),t._v(" "),t.activity.order?e("span",[t._v(" (#"+t._s(t.activity.order)+")")]):t._e()]),t._v(" "),e("i",{staticClass:"fas fa-sm fa-minus-circle",on:{click:function(){return t.toggleActivity(t.activityId)}}})])};c._withStripped=!0;var r=i(3851),n=i(1927);function a(t,e){var i=Object.keys(t);if(Object.getOwnPropertySymbols){var c=Object.getOwnPropertySymbols(t);e&&(c=c.filter((function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),i.push.apply(i,c)}return i}function s(t){for(var e,i=1;i<arguments.length;i++)e=null==arguments[i]?{}:arguments[i],i%2?a(Object(e),!0).forEach((function(i){o(t,i,e[i])})):Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(e)):a(Object(e)).forEach((function(i){Object.defineProperty(t,i,Object.getOwnPropertyDescriptor(e,i))}));return t}function o(t,e,i){return e in t?Object.defineProperty(t,e,{value:i,enumerable:!0,configurable:!0,writable:!0}):t[e]=i,t}const l={name:"ShowActivity",props:["activityId"],data:function(){return{}},computed:{activity:function(){return n.Z.query().with("meta").find(this.activityId)}},methods:s(s({},(0,r.nv)(["toggleActivity"])),{},{toggleEdit:function(){this.activity.meta.to_delete&&this.toggleActivity(this.activityId),this.$emit("toggle-edit")}})};const u=(0,i(1900).Z)(l,c,[],!1,null,null,null).exports}}]);
