"use strict";(self.webpackChunkpackage_edit=self.webpackChunkpackage_edit||[]).push([[555],{5555:(r,t,e)=>{e.r(t),e.d(t,{default:()=>n});var s=function(){var r=this,t=r._self._c;return t("div",{staticClass:"attribution-errors-block"},[t("ul",r._l(r.errors,(function(e){return t("li",[r._v(r._s(e))])})),0)])};s._withStripped=!0;const l={name:"Errors",props:["errors"]};const n=(0,e(1900).Z)(l,s,[],!1,null,null,null).exports}}]);
