{% extends "user/read.html" %}

{% block subtitle %}{{ _('Dataset Contributions') }} {{ g.template_title_delimiter }} {{ super() }}{% endblock %}

{% block primary_content_inner %}
<h1 class="hide-heading">
    {% block page_heading -%}
    {{ _('Contributions') }}
    {%- endblock %}
</h1>
{% if contributions %}
{% snippet 'snippets/contribution_list.html', contributions=contributions %}
{% endif %}
{% endblock %}
